// --------------------------------------------------------------------
// Copyright (c) 2005 by Terasic Technologies Inc. 
// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// --------------------------------------------------------------------
//           
//                     Terasic Technologies Inc
//                     356 Fu-Shin E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------
//
// Major Functions:	DE2 SD-Card Music-Player Reference Design
//
// --------------------------------------------------------------------
//
// Revision History :
// --------------------------------------------------------------------
//   Ver  :| Author            :| Mod. Date :| Changes Made:
//   V1.0 :| Johnny Chen       :| 05/08/19  :| Initial Revision
//   V1.1 :| Johnny Chen       :| 05/11/16  :| Add to FLASH Address FL_ADDR[21:20]
// --------------------------------------------------------------------

// Integrated with MP3 Code by Henry Ko on May 19, 2006

`define STATE_WRITE_WAIT_IDLE				'd0
`define STATE_WRITE_FIRST_BLOCK				'd1
`define STATE_WRITE_WAIT_REQUEST_CONFIRM	'd2
`define STATE_WRITE_WAIT_WRITE_FINISH		'd3
`define STATE_WRITE_CHECK_WRITE_REQUEST		'd4

`define STATE_READ_WAIT_FIRST_BLOCK			'd0
`define STATE_READ_ONE_BLOCK				'd1
`define STATE_READ_WAIT_NEXT_BLOCK			'd2

module DE2_SD_Card_Audio
	(
		//	Onboard Oscilator	 
		OSC_27,
		OSC_50,
		//	Push Button
		KEY,
		//	DPDT Switch
		DPDT_SW,
		//	7-SEG Dispaly
		HEX0,
		HEX1,
		HEX2,
		HEX3,
		HEX4,
		HEX5,
		HEX6,
		HEX7,
		//	LED
		LED_GREEN,
		LED_RED,
		//	UART
		UART_TXD,
		UART_RXD,
		//	SDRAM Interface
		DRAM_DQ,	//	16 Bits
		DRAM_ADDR,	//	12 Bits
		DRAM_LDQM,
		DRAM_UDQM,
		DRAM_WE_N,
		DRAM_CAS_N,
		DRAM_RAS_N,
		DRAM_CS_N,
		DRAM_BA_0,
		DRAM_BA_1,
		DRAM_CLK,
		DRAM_CKE,	
		//	Flash Interface
		FL_DQ,		//	8 Bits
		FL_ADDR,	//	22 Bits
		FL_WE_N,
		FL_RST_N,
		FL_OE_N,
		FL_CE_N,
		//	SRAM Interface
		SRAM_DQ,	//	16 Bits
		SRAM_ADDR,	//	18 Bits
		SRAM_UB_N,
		SRAM_LB_N,
		SRAM_WE_N,
		SRAM_CE_N,
		SRAM_OE_N,
		//	ISP1362 Interface
		OTG_DATA,	//	16 Bits
		OTG_ADDR,	//	2 Bits
		OTG_CS_N,
		OTG_RD_N,
		OTG_WR_N,
		OTG_RST_N,
		OTG_FSPEED,	//	USB Full Speed,	0 = Enable, Z = Disable
		OTG_LSPEED,	//	USB Low Speed, 	0 = Enable, Z = Disable
		OTG_INT0,
		OTG_INT1,
		OTG_DREQ0,
		OTG_DREQ1,
		OTG_DACK0_N,
		OTG_DACK1_N,
		//	Audio
		AUD_ADCLRCK,					//	Audio CODEC ADC LR Clock
		AUD_ADCDAT,						//	Audio CODEC ADC Data
		AUD_DACLRCK,					//	Audio CODEC DAC LR Clock
		AUD_DACDAT,						//	Audio CODEC DAC Data
		AUD_BCLK,						//	Audio CODEC Bit-Stream Clock
		AUD_XCK,						//	Audio CODEC Chip Clock
		//	LCD Module 16X2
		LCD_ON,		//	LCD ON/OFF
		LCD_BLON,	//	LCD Back Light ON/OFF
		LCD_RW,
		LCD_EN,
		LCD_RS,
		LCD_DATA,
		//	SD_Card Interface
		SD_DAT,
		SD_DAT3,
		SD_CMD,
		SD_CLK,
		//	I2C
		I2C_DATA,
		I2C_CLK,
		//	PS2
		PS2_DATA,
		PS2_CLK,
		//	TV
		TD_RESET
	);

//	On Board Oscilator
input			OSC_27;
input			OSC_50;
//	Push Button
input	[3:0]	KEY;
//	DPDT Switch
input	[17:0]	DPDT_SW;
//	7-SEG Dispaly
output	[6:0]	HEX0;
output	[6:0]	HEX1;
output	[6:0]	HEX2;
output	[6:0]	HEX3;
output	[6:0]	HEX4;
output	[6:0]	HEX5;
output	[6:0]	HEX6;
output	[6:0]	HEX7;
//	LED
output	[8:0]	LED_GREEN;
output	[17:0]	LED_RED;
//	UART
output			UART_TXD;
input			UART_RXD;
//	SDRAM Interface
inout	[15:0]	DRAM_DQ;	// 16 Bits
output	[11:0]	DRAM_ADDR;	// 12 Bits
output			DRAM_LDQM;
output			DRAM_UDQM;
output			DRAM_WE_N;
output			DRAM_CAS_N;
output			DRAM_RAS_N;
output			DRAM_CS_N;
output			DRAM_BA_0;
output			DRAM_BA_1;
output			DRAM_CLK;
output			DRAM_CKE;
//	Flash Interface
inout	[7:0]	FL_DQ;		//	8 Bits
output	[21:0]	FL_ADDR;	//	22 Bits
output			FL_WE_N;
output			FL_RST_N;
output			FL_OE_N;
output			FL_CE_N;
//	SRAM Interface
inout	[15:0]	SRAM_DQ;	//	16 Bits
output	[17:0]	SRAM_ADDR;	//	18 Bits
output			SRAM_UB_N;
output			SRAM_LB_N;
output			SRAM_WE_N;
output			SRAM_CE_N;
output			SRAM_OE_N;
//	ISP1362 Interface
inout	[15:0]	OTG_DATA;
output	[1:0]	OTG_ADDR;
output			OTG_WR_N;
output			OTG_RD_N;
output			OTG_CS_N;
output			OTG_RST_N;
output			OTG_FSPEED;		//	USB Full Speed,	0 = Enable, Z = Disable
output			OTG_LSPEED;		//	USB Low Speed, 	0 = Enable, Z = Disable
input			OTG_INT0;
input			OTG_INT1;
input			OTG_DREQ0;
input			OTG_DREQ1;
output			OTG_DACK0_N;
output			OTG_DACK1_N;
//	Audio
input/*inout*/	AUD_ADCLRCK;			//	Audio CODEC ADC LR Clock
input			AUD_ADCDAT;				//	Audio CODEC ADC Data
input/*inout*/	AUD_DACLRCK;			//	Audio CODEC DAC LR Clock
output			AUD_DACDAT;				//	Audio CODEC DAC Data
input			AUD_BCLK;				//	Audio CODEC Bit-Stream Clock
output			AUD_XCK;				//	Audio CODEC Chip Clock

//	LCD Module 16X2
inout	[7:0]	LCD_DATA;
output			LCD_ON;		//	LCD ON/OFF
output			LCD_BLON;	//	LCD Back Light ON/OFF
output			LCD_RW;
output			LCD_EN;
output			LCD_RS;
//	SD_Card Interface
inout			SD_DAT;
inout			SD_DAT3;
inout			SD_CMD;
output			SD_CLK;
//	I2C
inout			I2C_DATA;
output			I2C_CLK;
//	PS2
input		 	PS2_DATA;
input			PS2_CLK;
//	TV
output			TD_RESET;


//////////////////////////////////////////////////////////
wire	[31:0]	mSEG7_HEX;
wire	[15:0]	DATA_to_OTG;
wire	[15:0]	DATA_to_SRAM;
//	CPU
wire			CPU_RESET;

wire GLOBAL_RESETN;

// Dual Port
reg[9:0] 		DP_RAM_READ_ADDRESS;
wire[15:0] 		DP_RAM_READ_CH0_DATA;
wire[15:0] 		DP_RAM_READ_CH1_DATA;
wire[31:0] 		DP_RAM_WRITE_DATA;
wire			DP_RAM_WRITE_IDLE;
wire			DP_RAM_WRITE_REQUEST;
wire[3:0]		NIOS_CONTROL_SIGNALS;

wire 			WRITE_REQUEST;

// For I2C and Audio
wire [23:0] I2C_SETUP_DATA;
wire I2C_clock;
wire I2C_GO;
wire I2C_END;
wire I2C_ACK;
reg AUD_DACLRCK_buffer;
wire AUD_CTRL_CLK_18MHz;
wire AUD_CTRL_CLK_17MHz;
//wire AUD_CTRL_CLK_12MHz;
wire [7:0] VOLUME;
wire Volume_clock;

wire [1:0] SAMPLE_FREQUENCY;
wire [1:0] SAMPLE_FREQUENCY_SEL;
reg [15:0] DP_RAM_DATA_Buffer;
reg [31:0] bit_shift;

wire CODEC_READY;
wire NIOS_READY;

// For MP3
wire CLOCK_27MHZ;
wire HUFF_clock;
wire MAC_clock;

//	16*2 LCD Module
assign	LCD_ON		=	1'b1;	//	LCD ON
assign	LCD_BLON	=	1'b1;	//	LCD Back Light	
assign	FL_RST_N	=	1'b1;
assign	FL_ADDR[21:20]	=	2'b00;	//	Fixed 1M-Byte FLASH

//	Set SD Card to SD Mode
assign	SD_DAT3	=	1'b1;

//	Bypress 27 MHz
assign	TD_RESET	=	1'b1;

assign GLOBAL_RESETN = DPDT_SW[17] & NIOS_CONTROL_SIGNALS[3];

SDRAM_PLL 	PLL1	(.inclk0(OSC_50),.c0(DRAM_CLK));

//	Audio
//AUDIO_PLL  PLL2 	(.inclk0(OSC_27), .c0(AUD_CTRL_CLK_12MHz), .c1(CLOCK_27MHZ));
AUDIO_PLL2  PLL2 	(.inclk0(OSC_27), .c0(AUD_CTRL_CLK_18MHz), .c1(AUD_CTRL_CLK_17MHz), .c2(CLOCK_27MHZ));


Reset_Delay	rst		(OSC_50,CPU_RESET);

nios_0		u0	(
                // 1) global signals:
                 .clk(OSC_50),
                 .reset_n(CPU_RESET),

                // the_lcd_16207_0
                 .LCD_E_from_the_lcd_16207_0(LCD_EN),
                 .LCD_RS_from_the_lcd_16207_0(LCD_RS),
                 .LCD_RW_from_the_lcd_16207_0(LCD_RW),
                 .LCD_data_to_and_from_the_lcd_16207_0(LCD_DATA),

                // the_SEG7_Display
                 .oSEG0_from_the_SEG7_Display(HEX0),
                 .oSEG1_from_the_SEG7_Display(HEX1),
                 .oSEG2_from_the_SEG7_Display(HEX2),
                 .oSEG3_from_the_SEG7_Display(HEX3),
                 .oSEG4_from_the_SEG7_Display(HEX4),
                 .oSEG5_from_the_SEG7_Display(HEX5),
                 .oSEG6_from_the_SEG7_Display(HEX6),
                 .oSEG7_from_the_SEG7_Display(HEX7),

                // the_ISP1362
                 .OTG_ADDR_from_the_ISP1362(OTG_ADDR),
                 .OTG_CS_N_from_the_ISP1362(OTG_CS_N),
                 .OTG_DACK0_N_from_the_ISP1362(OTG_DACK0_N),
                 .OTG_DACK1_N_from_the_ISP1362(OTG_DACK1_N),
                 .OTG_DATA_to_and_from_the_ISP1362(OTG_DATA),
                 .OTG_DREQ0_to_the_ISP1362(OTG_DREQ0),
                 .OTG_DREQ1_to_the_ISP1362(OTG_DREQ1),
                 .OTG_INT0_to_the_ISP1362(OTG_INT0),
                 .OTG_INT1_to_the_ISP1362(OTG_INT1),
                 .OTG_RD_N_from_the_ISP1362(OTG_RD_N),
                 .OTG_RST_N_from_the_ISP1362(OTG_RST_N),
                 .OTG_WR_N_from_the_ISP1362(OTG_WR_N),
                 .OTG_FSPEED_from_the_ISP1362(OTG_FSPEED),
                 .OTG_LSPEED_from_the_ISP1362(OTG_LSPEED),
                 .iFSPEED_to_the_ISP1362(1'bz),
                 .iLSPEED_to_the_ISP1362(1'bz),

                // the_led_green
                 .out_port_from_the_led_green(LED_GREEN),

                // the_led_red
//                 .out_port_from_the_led_red(LED_RED),
				.out_port_from_the_led_red(),

                // the_switch_pio
                 .in_port_to_the_switch_pio(DPDT_SW),

                // the_button_pio
                 .in_port_to_the_button_pio(KEY),

                // the_SD_CLK
                 .out_port_from_the_SD_CLK(SD_CLK),

                // the_SD_CMD
                 .bidir_port_to_and_from_the_SD_CMD(SD_CMD),

                // the_SD_DAT
                 .bidir_port_to_and_from_the_SD_DAT(SD_DAT),

                // the_dp_ram_write_data
                .out_port_from_the_dp_ram_write_data(DP_RAM_WRITE_DATA),

				// the_dp_ram_write_idle
				.out_port_from_the_dp_ram_write_idle(DP_RAM_WRITE_IDLE),
				
				// the_dp_ram_write_request
				.in_port_to_the_dp_ram_write_request(WRITE_REQUEST),
				
				// control signals from nios				
  		        .out_port_from_the_nios_control_signals(NIOS_CONTROL_SIGNALS),

				// Codec ready signal
  		        .in_port_to_the_codec_ready(CODEC_READY),

                // the_sdram_0
                 .zs_addr_from_the_sdram_0(DRAM_ADDR),
                 .zs_ba_from_the_sdram_0({DRAM_BA_1,DRAM_BA_0}),
                 .zs_cas_n_from_the_sdram_0(DRAM_CAS_N),
                 .zs_cke_from_the_sdram_0(DRAM_CKE),
                 .zs_cs_n_from_the_sdram_0(DRAM_CS_N),
                 .zs_dq_to_and_from_the_sdram_0(DRAM_DQ),
                 .zs_dqm_from_the_sdram_0({DRAM_UDQM,DRAM_LDQM}),
                 .zs_ras_n_from_the_sdram_0(DRAM_RAS_N),
                 .zs_we_n_from_the_sdram_0(DRAM_WE_N),

                // the_sram_0
                 .SRAM_ADDR_from_the_sram_0(SRAM_ADDR),
                 .SRAM_CE_N_from_the_sram_0(SRAM_CE_N),
                 .SRAM_DQ_to_and_from_the_sram_0(SRAM_DQ),
                 .SRAM_LB_N_from_the_sram_0(SRAM_LB_N),
                 .SRAM_OE_N_from_the_sram_0(SRAM_OE_N),
                 .SRAM_UB_N_from_the_sram_0(SRAM_UB_N),
                 .SRAM_WE_N_from_the_sram_0(SRAM_WE_N),

                // the_tri_state_bridge_0_avalon_slave
                 .select_n_to_the_cfi_flash_0(FL_CE_N),
                 .tri_state_bridge_0_address(FL_ADDR),
                 .tri_state_bridge_0_data(FL_DQ),
                 .tri_state_bridge_0_readn(FL_OE_N),
                 .write_n_to_the_cfi_flash_0(FL_WE_N),

                // the_uart_0
                 .rxd_to_the_uart_0(UART_RXD),
                 .txd_from_the_uart_0(UART_TXD)
              );

assign NIOS_READY = NIOS_CONTROL_SIGNALS[0];

// For serializing data to I2C
i2c I2C_controller (
	.CLOCK(I2C_clock),
	.I2C_SCLK(I2C_CLK),//I2C CLOCK
 	.I2C_SDAT(I2C_DATA),//I2C DATA
	.I2C_DATA(I2C_SETUP_DATA),//DATA:[SLAVE_ADDR,SUB_ADDR,DATA]
	.GO(I2C_GO),      //GO transfor
	.END(I2C_END),     //END transfor 
	.W_R(1'b0),     //W_R
	.ACK(I2C_ACK),      //ACK
	.RESETN_I(GLOBAL_RESETN),
	//TEST
	.SD_COUNTER(),
	.SDO()
);

// Manual selection of sampling frequency
assign SAMPLE_FREQUENCY_SEL = (DPDT_SW[16] || DPDT_SW[0]) ? 2'b00 : SAMPLE_FREQUENCY;

// This select the input frequency for the audio codec for normal mode for different sampling frequency
assign AUD_XCK = (SAMPLE_FREQUENCY_SEL == 2'b00) ? AUD_CTRL_CLK_17MHz : AUD_CTRL_CLK_18MHz;

// For master mode
//assign AUD_XCK = AUD_CTRL_CLK_12MHz;

// This provide a roughly 1 Hz clock for the volume control
assign Volume_clock = LED_RED[1];

// For fast forwarding using a switch
assign WRITE_REQUEST = DPDT_SW[0] ? 1'b1 : DP_RAM_WRITE_REQUEST;

// For providing the setup instructions for the codec
audio audio_controller (
	.CLOCK_I(I2C_clock),
	.VOL_CLOCK_I(Volume_clock),
	.DATA_O(I2C_SETUP_DATA),
	.END_I(I2C_END),
	.RESETN_I(GLOBAL_RESETN),
	.GO_O(I2C_GO),
	.ACK_I(I2C_ACK),
	.FREQUENCY_I(SAMPLE_FREQUENCY_SEL),
//	.VOLUME_UP_I(NIOS_CONTROL_SIGNALS[2]),
//	.VOLUME_DOWN_I(NIOS_CONTROL_SIGNALS[1]),
	.LEFT_VOLUME_UP_I(DPDT_SW[15]),
	.LEFT_VOLUME_DOWN_I(DPDT_SW[14]),
	.RIGHT_VOLUME_UP_I(DPDT_SW[13]),
	.RIGHT_VOLUME_DOWN_I(DPDT_SW[12]),
	.CODEC_READY_O(CODEC_READY),
	.VOLUME_O(VOLUME)
);

// For serializing data to the audio codec
always @ (negedge AUD_BCLK or negedge GLOBAL_RESETN) begin
	if (GLOBAL_RESETN == 1'b0) begin
		DP_RAM_READ_ADDRESS <= 10'd512;
		DP_RAM_DATA_Buffer <= 16'd0;
	end else begin
		case (bit_shift)
		32'h40000000: begin
			if (NIOS_READY == 1'b1 && AUD_DACLRCK == 1'b0) begin
				DP_RAM_READ_ADDRESS <= DP_RAM_READ_ADDRESS + 10'd1;
			end
			DP_RAM_DATA_Buffer <= DP_RAM_DATA_Buffer << 1;
		end
		32'h80000000: begin
			if (AUD_DACLRCK == 1'b1) begin
				DP_RAM_DATA_Buffer <= DP_RAM_READ_CH1_DATA;
			end else begin
				DP_RAM_DATA_Buffer <= DP_RAM_READ_CH0_DATA;
			end
		end
		default: begin
			DP_RAM_DATA_Buffer <= DP_RAM_DATA_Buffer << 1;
		end
		endcase
	end
end

// For counting the number of bits that are shifted to the codec
always @ (posedge AUD_BCLK or negedge GLOBAL_RESETN) begin
	if (GLOBAL_RESETN == 1'b0) begin
		AUD_DACLRCK_buffer <= 1'b0;
		bit_shift <= 32'h00000001;
	end else begin
		AUD_DACLRCK_buffer <= AUD_DACLRCK;
		if (AUD_DACLRCK_buffer != AUD_DACLRCK) begin
			bit_shift <= 32'h00000001;
		end	else begin
			bit_shift <= bit_shift << 1;
		end
	end
end

// Serial output to the codec
assign AUD_DACDAT = NIOS_READY && !DPDT_SW[1] ? DP_RAM_DATA_Buffer[15] : 1'b0;

// The MP3 code
mp3_top MP3_core (
	.HUFF_CLOCK_I(HUFF_clock),
	.MAC_CLOCK_I(MAC_clock),
	.RESETN_I(GLOBAL_RESETN),
	
	// For NIOS MP3 data Input
	.NIOS_WRITE_DATA_CLK_I(OSC_50),
	.NIOS_WRITE_DATA_ADDRESS_I(DP_RAM_WRITE_DATA[26:17]),
	.NIOS_WRITE_DATA_I(DP_RAM_WRITE_DATA[16:1]),
	.NIOS_WRITE_DATA_EN_I(DP_RAM_WRITE_DATA[0]),
	.NIOS_WRITE_DATA_REQUEST_O(DP_RAM_WRITE_REQUEST),
	.NIOS_WRITE_DATA_IDLE_I(DP_RAM_WRITE_IDLE),
		
	// For NIOS PCM sample output
	.NIOS_READ_DATA_CLK_I(AUD_BCLK),
	.NIOS_READ_DATA_ADDRESS_I(DP_RAM_READ_ADDRESS),
	.NIOS_CH0_READ_DATA_O(DP_RAM_READ_CH0_DATA),
	.NIOS_CH1_READ_DATA_O(DP_RAM_READ_CH1_DATA),
	
	.NIOS_SAMPLE_FREQUENCY_O(SAMPLE_FREQUENCY),
// For debug
	.USER_LED0_O(LED_RED[1]),
	.USER_LED1_O(LED_RED[2])
);

// Clock divider circuit
clock_gen clock_divider (
	.CLOCK_27MHZ_I(CLOCK_27MHZ),
	.CLOCK_50MHZ_I(OSC_50),
	.RESETN_I(GLOBAL_RESETN),
	.CLOCK_DIV_4_O(MAC_clock),
	.CLOCK_DIV_16_O(HUFF_clock),
	.CLOCK_DIV_500_O(I2C_clock)
);

// for debug
reg [20:0] counter;
always @ (posedge AUD_BCLK or negedge GLOBAL_RESETN) begin
	if (GLOBAL_RESETN == 1'b0) begin
		counter <= 21'd0;
	end else begin
		counter <= counter + 21'd1;
	end
end
assign LED_RED[0] = counter[20];
assign LED_RED[10:3] = VOLUME;
assign LED_RED[11] = DP_RAM_WRITE_REQUEST;
assign LED_RED[12] = DP_RAM_WRITE_IDLE;
assign LED_RED[14:13] = SAMPLE_FREQUENCY_SEL;
assign LED_RED[15] = CODEC_READY;
assign LED_RED[16] = NIOS_READY;
assign LED_RED[17] = GLOBAL_RESETN;

// Switch
// 0: Fast forward
// 1: Force mute
// 2-11: Not used
// 12: Right channel volume down
// 13: Right channel volume up
// 14: Left channel volume down
// 15: Left channel volume up
// 16: Sampling frequency select
// 17: Global reset for hardware

// Buttons
// 0: Stop/Play
// 1: Fast forward
// 2: Rewind
// 3: NIOS reset
endmodule		

