
#include "basic_io.h"
#include "LCD.h"
#include "SD_Card.h"
//#include "Open_I2C.h"
#include "Test.h"
#include <math.h>

int Status;
UINT32 i, j;
int write_block_count;
int write_addr;
BYTE Buffer[1024]={0};
  
void System_init() {
  char Text1[16] = "SD MP3 Player!! ";
  char Text2[16] = "Initializing....";
  char Text3[16] = "Press button now";  
  unsigned char codec_ready;
  
  printf("Please wait while system is being initialized...\n");
  Status = 0;
    
  init_button_irq();
  printf("Button initialized...\n");
  
  SD_card_init();
  printf("SD card initialized...\n");
  
  LCD_Test();
  printf("LCD initialized...\n");
      
  LCD_Line1();
  //  Show Text to LCD
  LCD_Show_Text(Text1);
  //  Change Line2
  LCD_Line2();
  //  Show Text to LCD
  LCD_Show_Text(Text2);
  
  IOWR(LED_RED_BASE,0,0x0);
  IOWR(LED_GREEN_BASE,0,0x0);
  IOWR(SEG7_DISPLAY_BASE,0,0);
  
  IOWR(DP_RAM_WRITE_IDLE_BASE, 0, 1);
  IOWR(DP_RAM_WRITE_DATA_BASE, 0, 0);
  IOWR(NIOS_CONTROL_SIGNALS_BASE, 0, 0x0);
  
  // remove the reset signal from NIOS to hardware
  IOWR(NIOS_CONTROL_SIGNALS_BASE, 0, 0x8);
  
  printf("Waiting for audio codec...\n");
  do {
    codec_ready = IORD(CODEC_READY_BASE, 0);
  } while (codec_ready == 0);
  printf("Audio codec is now ready...\n");
   
  LCD_Line1();
  //  Show Text to LCD
  LCD_Show_Text(Text1);
  
  LCD_Line2();
  //  Show Text to LCD
  LCD_Show_Text(Text3);

  // Read the first block from SD card
  SD_read_lba(Buffer, 0, 2);
  j = 2;
  i = 0;
  write_addr = 0;
  write_block_count = 0;
 
  printf("System is ready now...\n");
  printf("Please push button 0 to start...\n");
}

int main(void)
{
  int write_request;
  
  System_init();
  
  while(1)
  {
    if (Status == 1) {
      write_request = IORD(DP_RAM_WRITE_REQUEST_BASE, 0);
      if (write_request == 1) {
//        printf("write requested...\n");
        write_block_count++;
        IOWR(DP_RAM_WRITE_IDLE_BASE, 0, 0);
        
        // Read 1024 bytes from SD card                       
        while (i < 1024) {
//          printf("j=%u i=%d block=%d Wdata %d: %04x\n", j, i, write_block_count, write_addr, Tmp1);
          IOWR(DP_RAM_WRITE_DATA_BASE, 0, ((write_addr & 0x3FF) << 17) | ((((Buffer[i]<<8)|(Buffer[i+1])) & 0xFFFF) << 1) | (1 & 0x1));
          if (write_addr < 1023) ++write_addr;
          else write_addr = 0;
          i += 2;
        }
        // Clear the write-enable signal
        IOWR(DP_RAM_WRITE_DATA_BASE, 0, 0);
        
//        printf("write finished: block %d, addr=%d\n", write_block_count, write_addr);
        IOWR(DP_RAM_WRITE_IDLE_BASE, 0, 1);
        IOWR(SEG7_DISPLAY_BASE,0, (write_block_count & 0xFFFF) << 16 | ((j / 2048) & 0xFFFF));                  
        
        // Prepare data for next write
        SD_read_lba(Buffer, j, 2);
        j+=2;
        i = 0;  
      }
//      printf("write: b=%d wc=%d i=%d wa=%d wd=%04X\n", write_block_count, write_counter, i, write_addr, Tmp1);      
    } else {
      msleep(500);
    }
  }
 
  return 0;
}

//-------------------------------------------------------------------------

