// 
//  MAC_MP3: A Low Energy Implementation of an Audio Decoder
//
//  Copyright (C) 2007 McMaster University
// 
//==============================================================================
// 
// This file is part of MAC_MP3
// 
// MAC_MP3 is distributed in the hope that it will be useful for further research,
// but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. MAC_MP3 is free; you can redistribute it
// and/or modify it provided that proper reference is provided to the authors. See
// the documents included in the "doc" folder for further details.
//
//==============================================================================


module eth_bank_0(
   resetn,
   clk_27mhz,
   memory_27mhz,
   start_fill,
   fill_done,
   ext_busy,
   range_end,

   DP_address,
   DP_enable,
   DP_data,

   // ZBT RAM INTERFACE                           
   MEMORY_BANK0_CLK_P, 		MEMORY_BANK1_CLK_P,
   MEMORY_BANK0_CLKEN_N, 	MEMORY_BANK1_CLKEN_N,
   MEMORY_BANK0_WEN_N,		MEMORY_BANK1_WEN_N,
   MEMORY_BANK0_WENA_N,		MEMORY_BANK1_WENA_N,
   MEMORY_BANK0_WENB_N,		MEMORY_BANK1_WENB_N,
   MEMORY_BANK0_WENC_N,		MEMORY_BANK1_WENC_N,
   MEMORY_BANK0_WEND_N,		MEMORY_BANK1_WEND_N,
   MEMORY_BANK0_ADV_LD_N, 	MEMORY_BANK1_ADV_LD_N,
   MEMORY_BANK0_OEN_N, 		MEMORY_BANK1_OEN_N,
   MEMORY_BANK0_CEN_N, 		MEMORY_BANK1_CEN_N,
   MEMORY_BANK0_ADDR_P,		MEMORY_BANK1_ADDR_P,
   MEMORY_BANK0_DATA_A_P, 	MEMORY_BANK1_DATA_A_P, 
   MEMORY_BANK0_DATA_B_P, 	MEMORY_BANK1_DATA_B_P,
   MEMORY_BANK0_DATA_C_P, 	MEMORY_BANK1_DATA_C_P,
   MEMORY_BANK0_DATA_D_P, 	MEMORY_BANK1_DATA_D_P,

   // ETHERNET INTERFACE
   TX_DATA_P,
   TX_ENABLE_P,
   TX_CLOCK_P,
   TX_ERROR_P,
   ENET_SLEW_P,
   RX_DATA_P,
   RX_DATA_VALID_P,
   RX_ERROR_P,
   RX_CLOCK_P,
   COLLISION_DETECTED_P,
   CARRIER_SENSE_P,
   PAUSE_P,
   MDIO_P,
   MDC_P,
   MDINIT_N,
   SSN_DATA_P
);
   
// SYSTEM INTERFACE
input                  resetn;
input                  clk_27mhz;
input                  memory_27mhz;
input                  start_fill;
output                 fill_done;
output                 ext_busy;
output                 range_end;

output    [9:0]        DP_address;
output                 DP_enable;
output    [15:0]       DP_data;

// ZBT RAM INTERFACE
output 					MEMORY_BANK0_CLK_P,		MEMORY_BANK1_CLK_P;		// ZBT RAM clock
output 					MEMORY_BANK0_CLKEN_N,	MEMORY_BANK1_CLKEN_N;	// ZBT RAM clock enable
output 					MEMORY_BANK0_WEN_N,		MEMORY_BANK1_WEN_N;		// ZBT RAM write enable
output 					MEMORY_BANK0_WENA_N,		MEMORY_BANK1_WENA_N;		// ZBT RAM byte A write enable
output 					MEMORY_BANK0_WENB_N,		MEMORY_BANK1_WENB_N;		// ZBT RAM byte B write enable
output 					MEMORY_BANK0_WENC_N,		MEMORY_BANK1_WENC_N;		// ZBT RAM byte C write enable
output 					MEMORY_BANK0_WEND_N,		MEMORY_BANK1_WEND_N;		// ZBT RAM byte D write enable
output 					MEMORY_BANK0_ADV_LD_N,	MEMORY_BANK1_ADV_LD_N;	// ZBT RAM address load-advance
output 					MEMORY_BANK0_OEN_N,		MEMORY_BANK1_OEN_N;		// ZBT RAM output enable
output              	MEMORY_BANK0_CEN_N,		MEMORY_BANK1_CEN_N;		// ZBT RAM chip enable
output   [18:0]     	MEMORY_BANK0_ADDR_P,		MEMORY_BANK1_ADDR_P;		// ZBT RAM address bus
inout    [7:0] 		MEMORY_BANK0_DATA_A_P,	MEMORY_BANK1_DATA_A_P;	// ZBT RAM data bus byte "A"
inout    [7:0] 		MEMORY_BANK0_DATA_B_P,	MEMORY_BANK1_DATA_B_P;	// ZBT RAM data bus byte "B"
inout    [7:0] 		MEMORY_BANK0_DATA_C_P,	MEMORY_BANK1_DATA_C_P;	// ZBT RAM data bus byte "C"
inout    [7:0] 		MEMORY_BANK0_DATA_D_P,	MEMORY_BANK1_DATA_D_P;	// ZBT RAM data bus byte "D"

// ETHERNET INTERFACE
output   [3:0]       TX_DATA_P;              // Ethernet transmission data
output               TX_ENABLE_P;			// Ehternet transmission enable
input                TX_CLOCK_P;			// Ethernet transmission clock
input                TX_ERROR_P;			// Ethernet transmission error
output   [1:0]       ENET_SLEW_P;			// Ethernet slew settings
input    [3:0]       RX_DATA_P;			// Ethernet receive data
input                RX_DATA_VALID_P;		// Ethernet data valid
input                RX_ERROR_P;			// Ethernet receive error
input                RX_CLOCK_P;			// Ethernet receive clock
input                COLLISION_DETECTED_P;	// Ethernet collision detected
input                CARRIER_SENSE_P;		// Ethernet carrier sense
output               PAUSE_P;             	// Ethernet pause
inout                MDIO_P;              	// Ethernet config data
output               MDC_P;               	// Ethernet config clock
input                MDINIT_N;            	// Ethernet config init
inout                SSN_DATA_P;          	// Silicon serial number access

reg                  fill_done;
reg                  range_end;
wire                 DP_enable;
reg      [3:0]       DP_enable_reg;
reg                  start_fill_d1;
reg                  start_fill_d2;
reg      [1:0]       state;
reg      [25:0]      address_counter;
reg      [23:0]      RAM_write_addr;
                 
wire                 sys_to_RAM_read;
wire                 sys_to_RAM_write;
wire     [23:0]	 sys_to_RAM_addr;
reg      [31:0]      RAM_to_sys_data;
wire     [31:0]      RAM0_to_sys_data, RAM1_to_sys_data;
reg      [31:0]      sys_to_RAM_data;

wire                 eth_to_sys_clock;
wire     [23:0]      eth_to_sys_addr;
wire                 eth_to_sys_rx;
wire     [31:0]      eth_to_sys_data;
wire                 eth_to_sys_write;
wire     [23:0]      eth_to_sys_range_out;

reg                  eth_to_sys_write_d1;
reg                  eth_to_sys_write_d2;
reg                  sys_to_RAM_write_d1;
wire                 sys_to_RAM0_write, sys_to_RAM1_write;

wire     [35:0]      chipscope_control;
wire                 chipscope_clock;
wire     [7:0]       chipscope_trigger;
wire     [95:0]      chipscope_data;

parameter ETH_EXT_WAIT = 2'h0;
parameter ETH_FILL_DP = 2'h1;
parameter ETH_EMPTY_PIPE = 2'h2;

assign ext_busy = eth_to_sys_rx;
assign sys_to_RAM_write = ~eth_to_sys_write_d1 & eth_to_sys_write_d2;
assign sys_to_RAM_read = (state == ETH_FILL_DP && address_counter[1:0] == 2'h0) ? 1'b1 : 1'b0;
assign sys_to_RAM_addr = (state == ETH_FILL_DP) ? address_counter[25:2] : RAM_write_addr;
assign DP_address = address_counter[10:1]-2;
assign DP_data = (address_counter[1] == 1'b0) ? RAM_to_sys_data[31:16] : RAM_to_sys_data[15:0];
assign DP_enable = DP_enable_reg[0];

/*
assign chipscope_clock = clk_27mhz;
assign chipscope_trigger = {  3'h0,
                              range_end,
						(sys_to_RAM_addr[18:0] == 19'h7FFFD) ? 1'b1 : 1'b0,
						(address_counter[20:0] == 21'h1FFFCF) ? 1'b1 : 1'b0,
                              eth_to_sys_rx,
						state == ETH_FILL_DP ? 1'b1 : 1'b0};
/*
assign chipscope_data[95:90] = 6'h00;
assign chipscope_data[89:88] = state;
assign chipscope_data[87] = DP_enable;
assign chipscope_data[86] = sys_to_RAM_read;
assign chipscope_data[85:82] = DP_enable_reg;
assign chipscope_data[81:58] = sys_to_RAM_addr;
assign chipscope_data[57:48] = DP_address;
assign chipscope_data[47:32] = DP_data;
assign chipscope_data[31:0] = RAM_to_sys_data;
/*
assign chipscope_data[95:79] = 17'h00000;
assign chipscope_data[78] = sys_to_RAM1_write;
assign chipscope_data[77] = sys_to_RAM0_write;
assign chipscope_data[76] = range_end;
assign chipscope_data[75] = (sys_to_RAM_addr[23:19] == 5'd1) ? 1'b1 : 1'b0;
assign chipscope_data[74] = (sys_to_RAM_addr[23:19] == 5'd0) ? 1'b1 : 1'b0;
assign chipscope_data[73:48] = address_counter;
assign chipscope_data[47:24] = eth_to_sys_range_out;
assign chipscope_data[23:0] = sys_to_RAM_addr;
/*
assign chipscope_data[95:86] = 10'h000;
assign chipscope_data[85] = DP_enable;
assign chipscope_data[84] = sys_to_RAM1_write;
assign chipscope_data[83] = sys_to_RAM0_write;
assign chipscope_data[82] = range_end;
assign chipscope_data[81:72] = DP_address;
assign chipscope_data[71:56] = DP_data;
assign chipscope_data[55:32] = sys_to_RAM_addr;
assign chipscope_data[31:0] = sys_to_RAM_data;
/**/

always @(posedge clk_27mhz or negedge resetn)
begin
   if (~resetn)
   begin
      start_fill_d1 <= 1'b0;
      start_fill_d2	<= 1'b0;
      address_counter <= 26'h0000000;
      DP_enable_reg <= 4'h0;
	 fill_done <= 1'b1;
	 state <= ETH_EXT_WAIT;
	 range_end <= 1'b0;
   end
   else
   begin
      start_fill_d1 <= start_fill;
	 start_fill_d2 <= start_fill_d1;
	 case (state)
	    ETH_EXT_WAIT : if (start_fill_d1 & ~start_fill_d2)
	                   begin
	                      state <= ETH_FILL_DP;
					  fill_done <= 1'b0;
                        end
                        else
				       state <= ETH_EXT_WAIT;
         ETH_FILL_DP :  begin
	                      if ((address_counter == 26'h3FFFFF) || (address_counter == {eth_to_sys_range_out,2'b11}))
								 begin
					  				range_end <= 1'b1; //address_counter <= 26'h000000;
									fill_done <= 1'b1;
								 end
                           else 
					     address_counter <= address_counter + 1;
                           
					  if (address_counter[0] == 1'b0)
					  	DP_enable_reg <= {1'b1, DP_enable_reg[3:1]};
                           else
					     DP_enable_reg <= {1'b0, DP_enable_reg[3:1]};
					  
					  if (ext_busy || address_counter[9:0] == 10'h3FF)
	                         state <= ETH_EMPTY_PIPE;
                           else
					     state <= ETH_FILL_DP;
                        end
         ETH_EMPTY_PIPE :  begin
	                      if (DP_enable_reg == 4'h0)
					  begin
					  	address_counter <= address_counter - 3;
						fill_done <= 1'b1;
						state <= ETH_EXT_WAIT;
                           end
					  else
					  begin 
					     address_counter <= address_counter + 1;
						state <= ETH_EMPTY_PIPE;
                           end                           
                           DP_enable_reg <= {1'b0, DP_enable_reg[3:1]};
                        end
      endcase
   end
end

always @(posedge clk_27mhz or negedge resetn)
begin
   if (~resetn)
   begin
      eth_to_sys_write_d1 <= 1'b0;
      eth_to_sys_write_d2 <= 1'b0;
      sys_to_RAM_data <= 32'h00000000;
      sys_to_RAM_write_d1 <= 1'b0;
   end
   else
   begin   
     eth_to_sys_write_d1 <= eth_to_sys_write;
     eth_to_sys_write_d2 <= eth_to_sys_write_d1;
	RAM_write_addr <= eth_to_sys_addr;
     sys_to_RAM_data <= eth_to_sys_data;
     sys_to_RAM_write_d1 <= sys_to_RAM_write;
   end
end
/*
icon i_icon
(
   .control0(chipscope_control)
);

ila i_ila
(
   .control(chipscope_control),
   .clk(chipscope_clock),
   .data(chipscope_data),
   .trig0(chipscope_trigger)
);
/**/

assign sys_to_RAM0_write = (sys_to_RAM_addr[23:19] == 5'd0) ? sys_to_RAM_write | sys_to_RAM_write_d1 : 1'b0;
assign sys_to_RAM1_write = (sys_to_RAM_addr[23:19] == 5'd1) ? sys_to_RAM_write | sys_to_RAM_write_d1 : 1'b0;
always @(RAM0_to_sys_data or RAM1_to_sys_data or sys_to_RAM_addr)
begin
	case(sys_to_RAM_addr[23:19])
		5'd0 : RAM_to_sys_data <= RAM0_to_sys_data;
		5'd1 : RAM_to_sys_data <= RAM1_to_sys_data;
		default : RAM_to_sys_data <= 32'h00000000;
	endcase
end

MEMORY_BANK0_RAM_CTRL MEMORY_BANK0_RAM_CTRL_1(
   .MEMORY_BANK0_CLK_P(MEMORY_BANK0_CLK_P),
   .MEMORY_BANK0_CLKEN_N(MEMORY_BANK0_CLKEN_N),
   .MEMORY_BANK0_WEN_N(MEMORY_BANK0_WEN_N),
   .MEMORY_BANK0_WENA_N(MEMORY_BANK0_WENA_N),
   .MEMORY_BANK0_WENB_N(MEMORY_BANK0_WENB_N),
   .MEMORY_BANK0_WENC_N(MEMORY_BANK0_WENC_N),
   .MEMORY_BANK0_WEND_N(MEMORY_BANK0_WEND_N),
   .MEMORY_BANK0_ADV_LD_N(MEMORY_BANK0_ADV_LD_N),
   .MEMORY_BANK0_OEN_N(MEMORY_BANK0_OEN_N),
   .MEMORY_BANK0_CEN_N(MEMORY_BANK0_CEN_N),
   .MEMORY_BANK0_ADDR_P(MEMORY_BANK0_ADDR_P),  
   .MEMORY_BANK0_DATA_A_P(MEMORY_BANK0_DATA_A_P),
   .MEMORY_BANK0_DATA_B_P(MEMORY_BANK0_DATA_B_P),
   .MEMORY_BANK0_DATA_C_P(MEMORY_BANK0_DATA_C_P),
   .MEMORY_BANK0_DATA_D_P(MEMORY_BANK0_DATA_D_P),
   .user_memory_bank0_clock(clk_27mhz),
   .user_memory_bank0_ram_clock(memory_27mhz),
   .user_memory_bank0_clocken(1'b1),
   .user_memory_bank0_load_addr(1'b1),
   .user_memory_bank0_read(sys_to_RAM_read),
   .user_memory_bank0_write(sys_to_RAM0_write),
   .user_memory_bank0_wea(1'b1),
   .user_memory_bank0_web(1'b1),
   .user_memory_bank0_wec(1'b1),
   .user_memory_bank0_wed(1'b1),
   .user_memory_bank0_ce(1'b1),
   .user_memory_bank0_addr(sys_to_RAM_addr[18:0]),
   .user_memory_bank0_read_data(RAM0_to_sys_data),
   .user_memory_bank0_write_data(sys_to_RAM_data)
);

MEMORY_BANK0_RAM_CTRL MEMORY_BANK1_RAM_CTRL_1(
   .MEMORY_BANK0_CLK_P(MEMORY_BANK1_CLK_P),
   .MEMORY_BANK0_CLKEN_N(MEMORY_BANK1_CLKEN_N),
   .MEMORY_BANK0_WEN_N(MEMORY_BANK1_WEN_N),
   .MEMORY_BANK0_WENA_N(MEMORY_BANK1_WENA_N),
   .MEMORY_BANK0_WENB_N(MEMORY_BANK1_WENB_N),
   .MEMORY_BANK0_WENC_N(MEMORY_BANK1_WENC_N),
   .MEMORY_BANK0_WEND_N(MEMORY_BANK1_WEND_N),
   .MEMORY_BANK0_ADV_LD_N(MEMORY_BANK1_ADV_LD_N),
   .MEMORY_BANK0_OEN_N(MEMORY_BANK1_OEN_N),
   .MEMORY_BANK0_CEN_N(MEMORY_BANK1_CEN_N),
   .MEMORY_BANK0_ADDR_P(MEMORY_BANK1_ADDR_P),  
   .MEMORY_BANK0_DATA_A_P(MEMORY_BANK1_DATA_A_P),
   .MEMORY_BANK0_DATA_B_P(MEMORY_BANK1_DATA_B_P),
   .MEMORY_BANK0_DATA_C_P(MEMORY_BANK1_DATA_C_P),
   .MEMORY_BANK0_DATA_D_P(MEMORY_BANK1_DATA_D_P),
   .user_memory_bank0_clock(clk_27mhz),
   .user_memory_bank0_ram_clock(memory_27mhz),
   .user_memory_bank0_clocken(1'b1),
   .user_memory_bank0_load_addr(1'b1),
   .user_memory_bank0_read(sys_to_RAM_read),
   .user_memory_bank0_write(sys_to_RAM1_write),
   .user_memory_bank0_wea(1'b1),
   .user_memory_bank0_web(1'b1),
   .user_memory_bank0_wec(1'b1),
   .user_memory_bank0_wed(1'b1),
   .user_memory_bank0_ce(1'b1),
   .user_memory_bank0_addr(sys_to_RAM_addr[18:0]),
   .user_memory_bank0_read_data(RAM1_to_sys_data),
   .user_memory_bank0_write_data(sys_to_RAM_data)
);

ETH_interface ETH_interface_1(
   .TX_DATA_P(TX_DATA_P),
   .TX_ENABLE_P(TX_ENABLE_P),
   .TX_CLOCK_P(TX_CLOCK_P),
   .TX_ERROR_P(TX_ERROR_P),
   .ENET_SLEW_P(ENET_SLEW_P),
   .RX_DATA_P(RX_DATA_P),
   .RX_DATA_VALID_P(RX_DATA_VALID_P),
   .RX_ERROR_P(RX_ERROR_P),
   .RX_CLOCK_P(RX_CLOCK_P),
   .COLLISION_DETECTED_P(COLLISION_DETECTED_P),
   .CARRIER_SENSE_P(CARRIER_SENSE_P),
   .PAUSE_P(PAUSE_P),
   .MDIO_P(MDIO_P),
   .MDC_P(MDC_P),
   .MDINIT_N(MDINIT_N),
   .SSN_DATA_P(SSN_DATA_P),
   .resetn_in(resetn),
   .clock_out(eth_to_sys_clock),
   .address_out(eth_to_sys_addr),
   .active_transmit_out(),
   .active_receive_out(eth_to_sys_rx),
   .error_out(),
   .start_send_in(1'b0),
   .dest_MAC_in(48'h000000000000),
   .type_in(16'h0000),
   .range_begin_in(24'h000000),
   .range_end_in(24'h000000),
   .data_in(32'h00000000),
   .read_out(),
   .src_MAC_out(),
   .type_out(),
   .type_valid_out(),
   .range_begin_out(),
   .range_end_out(eth_to_sys_range_out),
   .data_out(eth_to_sys_data),
   .write_out(eth_to_sys_write)
);

endmodule
/*   
module icon 
(
   control0
);
output [35:0] control0;
endmodule

module ila
(
   control,
   clk,
   data,
   trig0
);
input [35:0] control;
input        clk;
input [95:0] data;
input [7:0]  trig0;
endmodule
/**/
