// 
//  MAC_MP3: A Low Energy Implementation of an Audio Decoder
//
//  Copyright (C) 2007 McMaster University
// 
//==============================================================================
// 
// This file is part of MAC_MP3
// 
// MAC_MP3 is distributed in the hope that it will be useful for further research,
// but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. MAC_MP3 is free; you can redistribute it
// and/or modify it provided that proper reference is provided to the authors. See
// the documents included in the "doc" folder for further details.
//
//==============================================================================


#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <linux/if_packet.h>
#include <linux/if_ether.h>
#include <linux/if_arp.h>

#include <stdio.h>
#include <stdlib.h>

//#define ETH_FRAME_LEN 1514
#define packet_size 1024

int main(int argc, char *argv[])
{
   int i, j, k;
   long int data_length, range_begin, range_end;
   long int total_length, num_packets, last_packet_length;
   int s; /*socketdescriptor*/
   char temp_str[3];
   FILE *infile;

   if (argc < 2) {
     printf("Usage: sendmp3 <mp3_file>\n");
     exit(1);
   }   

   s = socket(AF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
   
   if (s == -1) printf("errorhandling ...\n");
   
   //Example 1.5. Send a RAW ethernet frame
   
   struct sockaddr_ll socket_address;                 /*target address*/
   void* buffer = (void*)malloc(ETH_FRAME_LEN);       /*buffer for ethernet frame*/
   unsigned char* etherhead = buffer;                 /*pointer to ethenet header*/
   unsigned char* data = buffer + 12;                 /*userdata in ethernet frame*/
   struct ethhdr *eh = (struct ethhdr *)etherhead;    /*another pointer to ethernet header*/
   int send_result = 0;
   
   unsigned char src_mac[6] = {0x00, 0x01, 0x02, 0xFA, 0x70, 0xAA};  /*our MAC address*/
   unsigned char dest_mac[6] = {0x5F, 0x43, 0xD4, 0xEA, 0x7B, 0x13}; /*other host MAC address*/
   
   /*prepare sockaddr_ll*//*RAW communication*/
   socket_address.sll_family   = PF_PACKET;	
   /*we don't use a protocoll above ethernet layer->just use anything here*/
   socket_address.sll_protocol = htons(ETH_P_IP);	
   /*index of the network device see full code later how to retrieve it*/
   socket_address.sll_ifindex  = 2;
   
   socket_address.sll_hatype   = ARPHRD_ETHER;     /*ARP hardware identifier is ethernet*/
   socket_address.sll_pkttype  = PACKET_OTHERHOST; /*target is another host*/
   socket_address.sll_halen    = ETH_ALEN;		    /*address length*/
   
   socket_address.sll_addr[0]  = 0x00;		       /*MAC - begin*/
   socket_address.sll_addr[1]  = 0x04;		
   socket_address.sll_addr[2]  = 0x75;
   socket_address.sll_addr[3]  = 0xC8;
   socket_address.sll_addr[4]  = 0x28;
   socket_address.sll_addr[5]  = 0xE5;             /*MAC - end*/
   
   socket_address.sll_addr[6]  = 0x00;/*not used*/
   socket_address.sll_addr[7]  = 0x00;/*not used*/
   
   /*set the frame header*/
   memcpy((void*)buffer, (void*)dest_mac, ETH_ALEN);
   memcpy((void*)(buffer+ETH_ALEN), (void*)src_mac, ETH_ALEN);
   eh->h_proto = 0x00;
   
   infile = fopen(argv[1], "rb");
   total_length = 2*2097152;

   num_packets = total_length / packet_size;
   printf("num_packets: %lu\n", num_packets);
   range_end = -1;
   for (i = 0; i < num_packets; i++)
   {
      data_length = packet_size/4;   // number of 32 bit words to be sent = number of addresses
	
	   // Type 
	   data[0] = 0x00;
	   data[1] = 0x00;
	
	   // Range begin
      range_begin = range_end + 1;
	   data[2] = (range_begin >> 16) & 0xFF;
	   data[3] = (range_begin >> 8) & 0xFF;
	   data[4] = range_begin & 0xFF;
	
	   // Range end
	   range_end = range_begin + data_length - 1;
	   data[5] = (range_end >> 16) & 0xFF;
	   data[6] = (range_end >> 8) & 0xFF; 
	   data[7] = range_end & 0xFF;        
	
	   // read packet data
	   for (j = 0; j < 4*data_length; j++)
	      if (feof(infile))
         {
            data_length = j/4;
            if (j % 4) 
               data_length++;
            if (data_length < 16)
               data_length = 16;
            for (k = j; k < 4*data_length; k++)
               data[k+8] = 0x00;
            range_end = range_begin + data_length - 1;
      	   data[5] = (range_end >> 16) & 0xFF;
	         data[6] = (range_end >> 8) & 0xFF; 
	         data[7] = range_end & 0xFF;        
            j = 4*data_length-1;
            i = num_packets-1;
         }
	      else
            fscanf(infile, "%c", &(data[j+8]));
	
	   /*send the packet*/
printf("data_length: %lu, range_begin %x %x %x, range_end %x %x %x\n", data_length, data[2], data[3], data[4], data[5], data[6], data[7]);
	   do {
	      send_result = sendto(s, buffer, 4*data_length+22, 0, (struct sockaddr*)&socket_address, sizeof(socket_address));
	   } while (send_result == -1);
   }
   fclose(infile);
}
